@@ -53,6 +53,19 @@ static void hinawa_snd_dg00x_init(HinawaSndDg00x *self)
 	return;
 }
 
+/**
+ * hinawa_snd_dg00x_new:
+ *
+ * Instantiate #HinawaSndDg00x object and return the instance.
+ *
+ * Returns: an instance of #HinawaSndDg00x.
+ * Since: 1.3.
+ */
+HinawaSndDg00x *hinawa_snd_dg00x_new(void)
+{
+	return g_object_new(HINAWA_TYPE_SND_DG00X, NULL);
+}
+
 /**
  * hinawa_snd_dg00x_open:
  * @self: A #HinawaSndUnit
@@ -78,14 +91,22 @@ void hinawa_snd_dg00x_open(HinawaSndDg00x *self, gchar *path, GError **exception
 	}
 }
 
+static void snd_dg00x_notify_msg(void *target, void *data, unsigned int length)
+{
+	HinawaSndDg00x *self = target;
+	struct snd_firewire_event_digi00x_message *event = data;
+
+	g_signal_emit(self, dg00x_sigs[DG00X_SIG_TYPE_MESSAGE], 0,
+		      event->message);
+}
+
 void hinawa_snd_dg00x_handle_msg(HinawaSndDg00x *self, const void *buf,
 				 unsigned int len)
 {
-	struct snd_firewire_event_digi00x_message *event =
-			(struct snd_firewire_event_digi00x_message *)buf;
+	int err = 0;
 
 	g_return_if_fail(HINAWA_IS_SND_DG00X(self));
 
-	g_signal_emit(self, dg00x_sigs[DG00X_SIG_TYPE_MESSAGE],
-		      0, event->message);
+	hinawa_context_schedule_notification(self, buf, len,
+					     snd_dg00x_notify_msg, &err);
 }
