@@ -79,6 +79,19 @@ static void hinawa_snd_dice_init(HinawaSndDice *self)
 	return;
 }
 
+/**
+ * hinawa_snd_dice_new:
+ *
+ * Instantiate #HinawaSndDice object and return the instance.
+ *
+ * Returns: an instance of #HinawaSndDice.
+ * Since: 1.3.
+ */
+HinawaSndDice *hinawa_snd_dice_new(void)
+{
+	return g_object_new(HINAWA_TYPE_SND_DICE, NULL);
+}
+
 /**
  * hinawa_snd_dice_open:
  * @self: A #HinawaSndUnit
@@ -181,6 +194,16 @@ end:
 	g_byte_array_unref(req_frame);
 }
 
+static void snd_dice_notify_notification(void *target, void *data,
+					 unsigned int length)
+{
+	HinawaSndDice *self = target;
+	struct snd_firewire_event_dice_notification *event = data;
+
+	g_signal_emit(self, dice_sigs[DICE_SIG_TYPE_NOTIFIED],
+		      0, event->notification);
+}
+
 void hinawa_snd_dice_handle_notification(HinawaSndDice *self,
 					 const void *buf, unsigned int len)
 {
@@ -188,6 +211,7 @@ void hinawa_snd_dice_handle_notification(HinawaSndDice *self,
 
 	GList *entry;
 	struct notification_waiter *waiter;
+	int err = 0;
 
 	struct snd_firewire_event_dice_notification *event =
 			(struct snd_firewire_event_dice_notification *)buf;
@@ -195,8 +219,8 @@ void hinawa_snd_dice_handle_notification(HinawaSndDice *self,
 	g_return_if_fail(HINAWA_IS_SND_DICE(self));
 	priv = hinawa_snd_dice_get_instance_private(self);
 
-	g_signal_emit(self, dice_sigs[DICE_SIG_TYPE_NOTIFIED],
-		      0, event->notification);
+	hinawa_context_schedule_notification(self, buf, len,
+					snd_dice_notify_notification, &err);
 
 	g_mutex_lock(&priv->mutex);
 
