@@ -85,6 +85,19 @@ static void hinawa_snd_efw_init(HinawaSndEfw *self)
 	return;
 }
 
+/**
+ * hinawa_snd_efw_new:
+ *
+ * Instantiate #HinawaSndEfw object and return the instance.
+ *
+ * Returns: an instance of #HinawaSndEfw.
+ * Since: 1.3.
+ */
+HinawaSndEfw *hinawa_snd_efw_new(void)
+{
+	return g_object_new(HINAWA_TYPE_SND_EFW, NULL);
+}
+
 /**
  * hinawa_snd_efw_open:
  * @self: A #HinawaSndUnit
