@@ -56,6 +56,19 @@ static void hinawa_snd_motu_init(HinawaSndMotu *self)
 	return;
 }
 
+/**
+ * hinawa_snd_motu_new:
+ *
+ * Instantiate #HinawaSndMotu object and return the instance.
+ *
+ * Returns: an instance of #HinawaSndMotu.
+ * Since: 1.3.
+ */
+HinawaSndMotu *hinawa_snd_motu_new(void)
+{
+	return g_object_new(HINAWA_TYPE_SND_MOTU, NULL);
+}
+
 /**
  * hinawa_snd_motu_open:
  * @self: A #HinawaSndUnit
@@ -81,14 +94,23 @@ void hinawa_snd_motu_open(HinawaSndMotu *self, gchar *path, GError **exception)
 	}
 }
 
-void hinawa_snd_motu_handle_notification(HinawaSndMotu *self,
-					 const void *buf, unsigned int len)
+static void snd_motu_notify_notification(void *target, void *data,
+					 unsigned int length)
 {
-	struct snd_firewire_event_motu_notification *event =
-			(struct snd_firewire_event_motu_notification *)buf;
-
-	g_return_if_fail(HINAWA_IS_SND_MOTU(self));
+	HinawaSndMotu *self = target;
+	struct snd_firewire_event_motu_notification *event = data;
 
 	g_signal_emit(self, motu_sigs[MOTU_SIG_TYPE_NOTIFIED], 0,
 		      event->message);
 }
+
+void hinawa_snd_motu_handle_notification(HinawaSndMotu *self,
+					 const void *buf, unsigned int len)
+{
+	int err = 0;
+
+	g_return_if_fail(HINAWA_IS_SND_MOTU(self));
+
+	hinawa_context_schedule_notification(self, buf, len,
+					     snd_motu_notify_notification, &err);
+}
