@@ -1,8 +1,10 @@
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 #include <string.h>
 #include <errno.h>
 
 #include "internal.h"
 #include "snd_tscm.h"
+#include "hinawa_sigs_marshal.h"
 
 /**
  * SECTION:snd_tscm
@@ -60,6 +62,19 @@ static void hinawa_snd_tscm_init(HinawaSndTscm *self)
 	return;
 }
 
+/**
+ * hinawa_snd_tscm_new:
+ *
+ * Instantiate #HinawaSndTscm object and return the instance.
+ *
+ * Returns: an instance of #HinawaSndTscm.
+ * Since: 1.3.
+ */
+HinawaSndTscm *hinawa_snd_tscm_new(void)
+{
+	return g_object_new(HINAWA_TYPE_SND_TSCM, NULL);
+}
+
 /**
  * hinawa_snd_tscm_open:
  * @self: A #HinawaSndUnit
@@ -114,26 +129,38 @@ const guint32 *hinawa_snd_tscm_get_state(HinawaSndTscm *self,
 	return priv->state;
 }
 
-void hinawa_snd_tscm_handle_control(HinawaSndTscm *self, const void *buf,
-				    unsigned int len)
+static void snd_tscm_notify_control(void *target, void *data,
+				    unsigned int length)
 {
-	struct snd_firewire_event_tascam_control *event =
-			(struct snd_firewire_event_tascam_control *)buf;
+	HinawaSndTscm *self = target;
+	struct snd_firewire_event_tascam_control *event = data;
 	struct snd_firewire_tascam_change *change;
 
-	g_return_if_fail(HINAWA_IS_SND_TSCM(self));
-
-	if (len < sizeof(event->type))
-		return;
-	len -= sizeof(event->type);
+	length -= sizeof(event->type);
 
 	change = event->changes;
-	while (len >= sizeof(*change)) {
+	while (length >= sizeof(*change)) {
 		g_signal_emit(self, tscm_sigs[TSCM_SIG_TYPE_CTL], 0,
 			      change->index,
 			      GUINT32_FROM_BE(change->before),
 			      GUINT32_FROM_BE(change->after));
 		++change;
-		len -= sizeof(*change);
+		length -= sizeof(*change);
 	}
 }
+
+void hinawa_snd_tscm_handle_control(HinawaSndTscm *self, const void *buf,
+				    unsigned int len)
+{
+	struct snd_firewire_event_tascam_control *event =
+			(struct snd_firewire_event_tascam_control *)buf;
+	int err = 0;
+
+	g_return_if_fail(HINAWA_IS_SND_TSCM(self));
+
+	if (len < sizeof(event->type))
+		return;
+
+	hinawa_context_schedule_notification(self, buf, len,
+					     snd_tscm_notify_control, &err);
+}
