@@ -12,7 +12,6 @@
 
 #include "internal.h"
 #include "hinawa_enums.h"
-#include "hinawa_context.h"
 
 /**
  * SECTION:snd_unit
@@ -29,6 +28,16 @@ G_DEFINE_QUARK("HinawaSndUnit", hinawa_snd_unit)
 	g_set_error(exception, hinawa_snd_unit_quark(), errno,		\
 		    "%d: %s", __LINE__, strerror(errno))
 
+struct _HinawaSndUnitPrivate {
+	int fd;
+	struct snd_firewire_get_info info;
+
+	gboolean streaming;
+
+	GSource *src;
+};
+G_DEFINE_TYPE_WITH_PRIVATE(HinawaSndUnit, hinawa_snd_unit, HINAWA_TYPE_FW_UNIT)
+
 typedef struct {
 	GSource src;
 	HinawaSndUnit *unit;
@@ -37,16 +46,6 @@ typedef struct {
 	unsigned int len;
 } SndUnitSource;
 
-struct _HinawaSndUnitPrivate {
-	int fd;
-	struct snd_firewire_get_info info;
-
-	gboolean streaming;
-
-	SndUnitSource *src;
-};
-G_DEFINE_TYPE_WITH_PRIVATE(HinawaSndUnit, hinawa_snd_unit, HINAWA_TYPE_FW_UNIT)
-
 enum snd_unit_prop_type {
 	SND_UNIT_PROP_TYPE_FW_TYPE = 1,
 	SND_UNIT_PROP_TYPE_CARD,
@@ -183,6 +182,19 @@ static void hinawa_snd_unit_init(HinawaSndUnit *self)
 	return;
 }
 
+/**
+ * hinawa_snd_unit_new:
+ *
+ * Instantiate #HinawaSndUnit object and return the instance.
+ *
+ * Returns: an instance of #HinawaSndUnit.
+ * Since: 1.3.
+ */
+HinawaSndUnit *hinawa_snd_unit_new(void)
+{
+	return g_object_new(HINAWA_TYPE_SND_UNIT, NULL);
+}
+
 /**
  * hinawa_snd_unit_open:
  * @self: A #HinawaSndUnit
@@ -282,22 +294,36 @@ void hinawa_snd_unit_ioctl(HinawaSndUnit *self, unsigned long request,
 		raise(exception, errno);
 }
 
-static void handle_lock_event(HinawaSndUnit *self,
-			      void *buf, unsigned int length)
+static void snd_unit_notify_lock(void *target, void *data, unsigned int length)
 {
-	struct snd_firewire_event_lock_status *event =
-			(struct snd_firewire_event_lock_status *)buf;
-
-	g_return_if_fail(HINAWA_IS_SND_UNIT(self));
+	HinawaSndUnit *self = target;
+	struct snd_firewire_event_lock_status *event = data;
 
 	g_signal_emit(self, snd_unit_sigs[SND_UNIT_SIG_TYPE_LOCK_STATUS], 0,
 		      event->status);
 }
 
+static void snd_unit_notify_disconnected(void *target, void *data,
+					 unsigned int length)
+{
+	HinawaSndUnit *self = target;
+
+	g_signal_emit_by_name(&self->parent_instance, "disconnected", NULL);
+}
+
+static void handle_lock_event(HinawaSndUnit *self,
+			      void *buf, unsigned int length)
+{
+	int err = 0;
+
+	hinawa_context_schedule_notification(self, buf, length,
+					     snd_unit_notify_lock, &err);
+}
+
 static gboolean prepare_src(GSource *src, gint *timeout)
 {
-	// Use 500 msec for safe cancellation of thread.
-	*timeout = 500;
+	// Use blocking poll(2) to save CPU usage.
+	*timeout = -1;
 
 	/* This source is not ready, let's poll(2) */
 	return FALSE;
@@ -306,32 +332,44 @@ static gboolean prepare_src(GSource *src, gint *timeout)
 static gboolean check_src(GSource *gsrc)
 {
 	SndUnitSource *src = (SndUnitSource *)gsrc;
-	HinawaSndUnit *unit = src->unit;
-	HinawaSndUnitPrivate *priv = hinawa_snd_unit_get_instance_private(unit);
 	GIOCondition condition;
-	GValue val = G_VALUE_INIT;
 
+	condition = g_source_query_unix_fd(gsrc, src->tag);
+	if (condition & G_IO_ERR) {
+		HinawaSndUnit *unit = src->unit;
+		if (unit != NULL) {
+			gboolean listening;
+			int err = 0;
+
+			// For emitting one signal.
+			g_object_get(G_OBJECT(&unit->parent_instance),
+				     "listening", &listening, NULL);
+
+			hinawa_snd_unit_unlisten(unit);
+
+			if (listening) {
+				hinawa_context_schedule_notification(unit,
+					NULL, 0, snd_unit_notify_disconnected,
+					&err);
+			}
+		}
+	}
+
+	// Don't go to dispatch if nothing available.
+	return !!(condition & G_IO_IN);
+}
+
+static gboolean dispatch_src(GSource *gsrc, GSourceFunc cb, gpointer user_data)
+{
+	SndUnitSource *src = (SndUnitSource *)gsrc;
+	HinawaSndUnit *unit = src->unit;
+	HinawaSndUnitPrivate *priv;
 	struct snd_firewire_event_common *common;
 	int len;
 
 	if (unit == NULL)
 		goto end;
-
-	condition = g_source_query_unix_fd((GSource *)src, src->tag);
-	if (condition & G_IO_ERR) {
-		/* For emitting one signal. */
-		g_value_init(&val, G_TYPE_BOOLEAN);
-		g_object_get_property(G_OBJECT(&unit->parent_instance),
-				      "listening", &val);
-		hinawa_snd_unit_unlisten(unit);
-		if (g_value_get_boolean(&val))
-			g_signal_emit_by_name(&unit->parent_instance,
-					      "disconnected", NULL);
-		goto end;
-	/* The other events never occur. */
-	} else if (!(condition & G_IO_IN)) {
-		goto end;
-	}
+	priv = hinawa_snd_unit_get_instance_private(unit);
 
 	len = read(priv->fd, src->buf, src->len);
 	if (len <= 0)
@@ -372,24 +410,66 @@ static gboolean check_src(GSource *gsrc)
 						    src->buf, len);
 #endif
 end:
-	/* Don't go to dispatch, then continue to process this source. */
-	return FALSE;
-}
-
-static gboolean dispatch_src(GSource *src, GSourceFunc callback,
-			     gpointer user_data)
-{
 	/* Just be sure to continue to process this source. */
-	return TRUE;
+	return G_SOURCE_CONTINUE;
 }
 
 static void finalize_src(GSource *gsrc)
 {
 	SndUnitSource *src = (SndUnitSource *)gsrc;
+	HinawaSndUnitPrivate *priv =
+				hinawa_snd_unit_get_instance_private(src->unit);
+
+	if (priv->streaming)
+		ioctl(priv->fd, SNDRV_FIREWIRE_IOCTL_UNLOCK, NULL);
 
 	g_free(src->buf);
 }
 
+static void snd_unit_create_source(HinawaSndUnit *self, GSource **gsrc,
+				   GError **exception)
+{
+	static GSourceFuncs funcs = {
+		.prepare	= prepare_src,
+		.check		= check_src,
+		.dispatch	= dispatch_src,
+		.finalize	= finalize_src,
+	};
+	HinawaSndUnitPrivate *priv;
+	SndUnitSource *src;
+
+	g_return_if_fail(HINAWA_IS_SND_UNIT(self));
+	priv = hinawa_snd_unit_get_instance_private(self);
+
+	*gsrc = g_source_new(&funcs, sizeof(SndUnitSource));
+
+	g_source_set_name(*gsrc, "HinawaSndUnit");
+	g_source_set_priority(*gsrc, G_PRIORITY_HIGH_IDLE);
+	g_source_set_can_recurse(*gsrc, TRUE);
+
+	// MEMO: allocate one page because we cannot assume the size of
+	// transaction frame.
+	src = (SndUnitSource *)(*gsrc);
+	src->len = sysconf(_SC_PAGESIZE);
+	src->buf = g_malloc(src->len);
+	if (src->buf == NULL) {
+		raise(exception, ENOMEM);
+		g_source_unref(*gsrc);
+		return;
+	}
+
+	src->unit = self;
+	src->tag = g_source_add_unix_fd(*gsrc, priv->fd, G_IO_IN);
+
+	// Check locked or not.
+	if (ioctl(priv->fd, SNDRV_FIREWIRE_IOCTL_LOCK, NULL) < 0) {
+		if (errno == EBUSY)
+			priv->streaming = true;
+		return;
+	}
+	ioctl(priv->fd, SNDRV_FIREWIRE_IOCTL_UNLOCK, NULL);
+}
+
 /**
  * hinawa_snd_unit_listen:
  * @self: A #HinawaSndUnit
@@ -399,70 +479,25 @@ static void finalize_src(GSource *gsrc)
  */
 void hinawa_snd_unit_listen(HinawaSndUnit *self, GError **exception)
 {
-	static GSourceFuncs funcs = {
-		.prepare	= prepare_src,
-		.check		= check_src,
-		.dispatch	= dispatch_src,
-		.finalize	= finalize_src,
-	};
 	HinawaSndUnitPrivate *priv;
-	void *buf;
-	unsigned int len;
-	GSource *src;
 
 	g_return_if_fail(HINAWA_IS_SND_UNIT(self));
 	priv = hinawa_snd_unit_get_instance_private(self);
 
-	/*
-	 * MEMO: allocate one page because we cannot assume the size of
-	 * transaction frame.
-	 */
-	len = sysconf(_SC_PAGESIZE);
-	buf = g_malloc(len);
-	if (buf == NULL) {
-		raise(exception, ENOMEM);
+	snd_unit_create_source(self, &priv->src, exception);
+	if (*exception != NULL)
 		return;
-	}
 
-	src = g_source_new(&funcs, sizeof(SndUnitSource));
-	if (src == NULL) {
-		raise(exception, ENOMEM);
-		g_free(buf);
-		return;
-	}
-
-	g_source_set_name(src, "HinawaSndUnit");
-	g_source_set_priority(src, G_PRIORITY_HIGH_IDLE);
-	g_source_set_can_recurse(src, TRUE);
-
-	((SndUnitSource *)src)->len = len;
-	((SndUnitSource *)src)->buf = buf;
-	((SndUnitSource *)src)->unit = self;
-	((SndUnitSource *)src)->tag =
-				g_source_add_unix_fd(src, priv->fd, G_IO_IN);
-
-	hinawa_context_add_src(HINAWA_CONTEXT_TYPE_SND, src, exception);
+	hinawa_context_add_src(priv->src, exception);
 	if (*exception != NULL) {
-		g_source_destroy(src);
+		g_source_unref(priv->src);
 		priv->src = NULL;
 		return;
 	}
 
-	priv->src = (SndUnitSource *)src;
-
 	hinawa_fw_unit_listen(&self->parent_instance, exception);
-	if (*exception != NULL) {
+	if (*exception != NULL)
 		hinawa_snd_unit_unlisten(self);
-		return;
-	}
-
-	/* Check locked or not. */
-	if (ioctl(priv->fd, SNDRV_FIREWIRE_IOCTL_LOCK, NULL) < 0) {
-		if (errno == EBUSY)
-			priv->streaming = true;
-		return;
-	}
-	ioctl(priv->fd, SNDRV_FIREWIRE_IOCTL_UNLOCK, NULL);
 }
 
 /**
@@ -474,26 +509,15 @@ void hinawa_snd_unit_listen(HinawaSndUnit *self, GError **exception)
 void hinawa_snd_unit_unlisten(HinawaSndUnit *self)
 {
 	HinawaSndUnitPrivate *priv;
-	GSource *gsrc;
 
 	g_return_if_fail(HINAWA_IS_SND_UNIT(self));
 	priv = hinawa_snd_unit_get_instance_private(self);
 
-	if (priv->src == NULL)
-		return;
-	gsrc = (GSource *)priv->src;
-
-	if (priv->streaming)
-		ioctl(priv->fd, SNDRV_FIREWIRE_IOCTL_UNLOCK, NULL);
-
-	g_source_remove_unix_fd(gsrc, priv->src->tag);
-
-	hinawa_context_remove_src(HINAWA_CONTEXT_TYPE_SND, gsrc);
-
-	g_source_destroy(gsrc);
-
-	g_free(priv->src);
-	priv->src = NULL;
+	if (priv->src != NULL) {
+		hinawa_context_remove_src(priv->src);
+		g_source_unref(priv->src);
+		priv->src = NULL;
+	}
 
 	hinawa_fw_unit_unlisten(&self->parent_instance);
 }
